@@ -10,12 +10,9 @@
 #ifndef CODEGENFACTORY_H
 #define CODEGENFACTORY_H
 
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qobject.h>
+#include "../umlnamespace.h"
 
 class CodeGenerator;
-class UMLDoc;
 
 /**
  * CodeGeneratorFactory lists gives access to the available code generators, their names 
@@ -27,21 +24,10 @@ class UMLDoc;
 class CodeGeneratorFactory {
 public:
 
-        CodeGeneratorFactory();
-        virtual ~CodeGeneratorFactory();
+    CodeGeneratorFactory();
+    virtual ~CodeGeneratorFactory();
 
-        /**
-         * Returns a QStringList containing the languages offered by this library
-         */
-        QStringList languagesAvailable();
-
-        /**
-         * Returns the name of the generator which implements language l
-         */
-        QString generatorName(const QString &l);
-
-//FIXMEnow        virtual QObject* createObject(QObject* parent, const char* name);
-        CodeGenerator* createObject(UMLDoc* parent, const char* name);
+    CodeGenerator* createObject(Uml::Programming_Language pl);
 };
 
 #endif //CODEGENFACTORY_H
